@@ -16,7 +16,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.3.8';
+    $VERSION = '1.3.9';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -94,7 +94,7 @@ sub add_defaults {
         'unsharp threshold' => 0.05,
         'cache options'     => TRUE,
         'restore window'    => TRUE,
-        'date offset'       => 0,
+        'document date'     => time,
         'pdf compression'   => 'auto',
         'quality'           => 75,
         'pages to scan'     => 1,
