@@ -226,7 +226,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '1.3.8';
+our $VERSION = '1.3.9';
 
 my ( $d, $d_sane, $logger, $tooltips );
 my $tolerance = 1;
@@ -489,7 +489,7 @@ sub INIT_INSTANCE {
     $framesp->add($vboxsp);
     $self->{combobsp} = Gtk2::ComboBox->new_text;
     $self->{combobsp}->signal_connect( changed =>
-          sub { $self->set_profile( $self->{combobsp}->get_active_text ) } );
+          sub { $self->set( 'profile', $self->{combobsp}->get_active_text ) } );
     $vboxsp->pack_start( $self->{combobsp}, FALSE, FALSE, 0 );
     my $hboxsp = Gtk2::HBox->new;
     $vboxsp->pack_end( $hboxsp, FALSE, FALSE, 0 );
@@ -638,10 +638,32 @@ sub SET_PROPERTY {
                 $widget->set_active(TRUE);
             }
             when ('paper') {
-                $self->{$name} = $newval;
-                if ( set_combobox_by_text( $self->{combobp}, $newval ) ) {
-                    $self->signal_emit( 'changed-paper', $newval );
+                if ( defined $newval ) {
+                    for ( @{ $self->{ignored_paper_formats} } ) {
+                        if ( $_ eq $newval ) {
+                            if ( defined $logger ) {
+                                $logger->info(
+                                    "Ignoring unsupported paper $newval");
+                                $logger->debug("Finished$msg");
+                            }
+                            return;
+                        }
+                    }
                 }
+                $callback = TRUE;
+                my $signal;
+                $signal = $self->signal_connect(
+                    'changed-paper' => sub {
+                        $self->signal_handler_disconnect($signal);
+                        my $paper =
+                          defined $newval ? $newval : $d->get('Manual');
+                        set_combobox_by_text( $self->{combobp}, $paper );
+                        if ( defined $logger ) {
+                            $logger->debug("Finished$msg");
+                        }
+                    }
+                );
+                $self->set_paper($newval);
             }
             when ('paper_formats') {
                 $self->{$name} = $newval;
@@ -1226,6 +1248,17 @@ sub remove_profile {
             $self->{combobsp}->remove_text($i);
             $self->signal_emit( 'removed-profile', $name );
         }
+        delete $self->{profiles}{$name};
+    }
+    return;
+}
+
+# Helper function to check the new value is different before adding the option
+# to the profile
+sub build_profile {
+    my ( $self, $profile, $option, $newval ) = @_;
+    if ( $option->{val} != $newval ) {
+        push @{$profile}, { $option->{name} => $newval };
     }
     return;
 }
